package org.simantics.databoard.tests;

import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.concurrent.ConcurrentHashMap;

import org.simantics.databoard.Bindings;
import org.simantics.databoard.annotations.Arguments;
import org.simantics.databoard.annotations.Referable;
import org.simantics.databoard.annotations.Union;
import org.simantics.databoard.binding.Binding;
import org.simantics.databoard.binding.reflection.BindingRequest;
import org.simantics.databoard.binding.reflection.ReflectionBindingFactory;
import org.simantics.databoard.type.Datatype;

public class Jotakin9 {

	public @Referable static class Foo {
		public List<String> list;
		public String[] array;
		public LinkedList<String> linkedList;		
		public HashMap<Integer, String> hashMap;
		public TreeMap<Integer, String> treeMap;
			
		public Foo() {			
		}
		
		public Foo(String[] list, int[] keys, String values[]) {
			this.list = new ArrayList<String>(list.length);
			this.linkedList = new LinkedList<String>();
			array = list.clone();
			for (int i=0; i<list.length; i++) {
				this.list.add( list[i] ); 
				this.linkedList.add( list[i] );
			}
			hashMap = new HashMap<Integer, String>();
			treeMap = new TreeMap<Integer, String>();
			for (int i=0; i<keys.length; i++) {
				hashMap.put( keys[i] , values[i] );
				treeMap.put( keys[i] , values[i] );
			}
		}
	}	
	
	public static class Bar {
		
		public @Arguments(List.class) List list;
		
	}
	
	@Union({Rectangle.class, Circle.class, Triangle.class})
	static class Shape {}	
	static class Rectangle extends Shape {}
	static class Circle extends Shape {}		
	static class Triangle extends Shape {}		
	
	static public class Example1 {
		
		public int x;

		public ArrayList<Integer> y;
		
		public TreeMap<Integer, Double> z;		
		
	}
	
	static enum abc {a,b,c}
	

	
	public static void main(String[] args) throws Exception {
		
		Binding c = Bindings.getBinding(Map.class, Integer.class, String.class);
		Map<Integer, String> map = (Map<Integer, String>) c.createRandom(5);
		System.out.println(c.toString(map));
		
		Binding d = Bindings.getBinding(List.class, Integer.class);
		List<Integer> list = (List<Integer>) d.createRandom(5);
		System.out.println(d.toString(list));
		
		Binding e = Bindings.getBinding(List.class, List.class, String.class);
		List<List<String>> listList = (List<List<String>>) e.createRandom(5);
		System.out.println(e.toString(listList));

		Binding b = Bindings.getBinding(Foo.class);
		b.createDefault();
		System.out.println(b.getClass().getSimpleName());
				
		Map<BindingRequest, Binding> typeRepository = new ConcurrentHashMap<BindingRequest, Binding>();
		ReflectionBindingFactory factory = new ReflectionBindingFactory(typeRepository);
		
		Datatype type = factory.construct( new BindingRequest( Example1.class ) ).type();
		
		System.out.println(type);
		
		Rectangle2D rect = new Rectangle2D.Double();
		Binding binding = Bindings.getBinding( rect.getClass() );
		byte[] data = binding.serializer().serialize(rect);
		
	}
	
}
