package org.simantics.databoard.tests;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.nio.charset.Charset;

import org.simantics.databoard.Bindings;
import org.simantics.databoard.binding.StringBinding;

/**
 *
 * @author Toni Kalajainen <toni.kalajainen@iki.fi>
 */
public class StringUTFTest {
	
	public static void main(String[] args) throws Exception {
		
		StringBinding b = Bindings.STRING;
		int c = 10000;
		String[] strs = new String[c];
		for (int i=0; i<c; i++) strs[i] = (String) b.createRandom(i);
		
				
		{
			ByteArrayOutputStream baos = new ByteArrayOutputStream( 10*1024*1024 );
			DataOutputStream dos = new DataOutputStream( baos );		
			System.gc();
			long x = System.nanoTime();
			for (int i=0; i<c; i++) {
				dos.writeUTF(strs[i]);
			}
			long y = System.nanoTime() - x;
			System.out.println("DataOutputStream.writeUTF: "+y);

			byte[] data = baos.toByteArray();
			DataInputStream dis = new DataInputStream( new ByteArrayInputStream( data ) );
			x = System.nanoTime();
			for (int i=0; i<c; i++) {
				dis.readUTF();
			}
			y = System.nanoTime() - x;
			System.out.println("DataOutputStream.readUTF : "+y);
		}
		
		{
			Charset cs = Charset.forName("UTF8");
			ByteArrayOutputStream baos = new ByteArrayOutputStream( 10*1024*1024 );
			DataOutputStream dos = new DataOutputStream( baos );		
			System.gc();
			long x = System.nanoTime();
			for (int i=0; i<c; i++) {
				byte[] data = strs[i].getBytes(cs);
				dos.writeInt(data.length);
				dos.write(data);
			}
			long y = System.nanoTime() - x;
			System.out.println("String#getBytes(cs)      : "+y);

			
			byte[] strData = new byte[1024];
			byte[] data = baos.toByteArray();
			DataInputStream dis = new DataInputStream( new ByteArrayInputStream( data ) );
			x = System.nanoTime();
			for (int i=0; i<c; i++) {
				int len = dis.readInt();
				dis.read(strData, 0, len);
				new String(strData, cs);
			}
			y = System.nanoTime() - x;
			System.out.println("new String(byte[], cs)   : "+y);
		}
		
		
	}

}
