/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.type;

import java.util.Set;

import org.simantics.databoard.util.IdentityPair;
import org.simantics.databoard.util.Limit;
import org.simantics.databoard.util.ObjectUtils;
import org.simantics.databoard.util.Range;

public class LongType extends NumberType {
	    
    public LongType() {}
    public LongType(String unit) {
    	this.unit = unit;
    }
    public LongType(String unit, Range range) {
    	this.unit = unit;
    	setRange(range);
    }    
    public LongType(String unit, String range) {
    	this.unit = unit;
    	setRange(range);
    }    
    
    public long minValue() {
    	if (range==null) return Long.MIN_VALUE;
    	Limit l = range.getLower();
    	long value = l.getValue().longValue();
    	if (l.isExclusive()) value++;
    	return value;
    }
    
    public long maxValue() {
    	if (range==null) return Long.MAX_VALUE;
    	Limit l = range.getUpper();
    	long value = l.getValue().longValue();
    	if (l.isExclusive()) value--;
    	return value;
    }    
    
    @Override
    protected boolean deepEquals(Object obj,
    		Set<IdentityPair<Datatype, Datatype>> compareHistory) {
		if (this==obj) return true;
		if (obj instanceof LongType == false) return false;
		LongType other = (LongType) obj;
		return 
		ObjectUtils.objectEquals(other.range, range) &&
		ObjectUtils.objectEquals(other.unit, unit);
    }

    @Override
    public int hashCode() {
    	return 0x367654 + 13*ObjectUtils.hashCode(range) + 7*ObjectUtils.hashCode(unit);
    }     

	@Override
	public void accept(Visitor1 v, Object obj) {
	    v.visit(this, obj);        
	}

	@Override
	public <T> T accept(Visitor<T> v) {
	    return v.visit(this);
	}	
	
}
